/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.pdfconv;

import org.jpedal.PdfDecoder;
import org.jpedal.io.ObjectStore;

import java.awt.image.BufferedImage;
import java.io.File;

import jcmdline.*;

public class PDFConv
{
    private static final String VERSION="0.1.4";
    
    private String _pdfName;
    private String _baseName;
    private String _targetDir="./output/";

    boolean _usePict=false;

    private DocumentWriter _documentWriter=null;
    private ImageWriter _imageWriter=null;

    public static void main(String[] args)
    {
        try
        {
            PDFConv pc=new PDFConv();
            pc.readParameters(args);
            pc.convert();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

    }
    
    public PDFConv()
    {
    }

    private void readParameters(String[] args) 
    {
        String helpText="This program reads a PDF file, and saves each page as an image (JPEG or PICT). Additionaly, a HTML or " +
            "BookMaker source file is written to reference all files.";

        FileParam sourceParam=new FileParam("source","the PDF input file",FileParam.IS_FILE/*&FileParam.IS_READABLE*/,FileParam.REQUIRED,FileParam.SINGLE_VALUED);
        StringParam baseNameParam=new StringParam("basename","the base name for all output files",false);
        IntParam widthParam=new IntParam("maxwidth","maximum width of a resulting image",50,1000,true);
        IntParam heigthParam=new IntParam("maxheigth","maximum heigth of a resulting image",50,1000,true);
        StringParam scaleFactorParam=new StringParam("scalefactor","how to scale the created images (float value)",true);
        FileParam targetDirParam=new FileParam("targetdir","where to write the resulting files",FileParam.IS_DIR&FileParam.IS_WRITEABLE,FileParam.OPTIONAL,FileParam.SINGLE_VALUED);
        IntParam qualityParam=new IntParam("quality","quality of a resulting JPEG image",10,100,true);
        BooleanParam pictParam=new BooleanParam("pict","write PICT files instead of JPEG, and create a BookMaker file instead of HTML");

        String sourceFile=null;
        String targetBaseName=null;
        String targetDir="";
        boolean writePict=false;

        CmdLineHandler cl =
            new VersionCmdLineHandler("V "+VERSION,
            new HelpCmdLineHandler(helpText,
                "run",
                "convert PDF files for showing on a Newton",
                new Parameter[] { pictParam, widthParam, heigthParam, scaleFactorParam, targetDirParam, qualityParam },
                new Parameter[] { sourceParam, baseNameParam },
                new PosixCmdLineParser()));
        cl.parse(args);
        
        if (pictParam.isTrue())
            writePict=true;
        targetDir=targetDirParam.getValue();
        sourceFile=sourceParam.getValue();
        targetBaseName=baseNameParam.getValue();
        
        if (null!=targetDir&&0!=targetDir.length())
        {
            targetDir=targetDir.replace('\\', '/');
            if (!targetDir.endsWith("/"))
                targetDir+="/";
            _targetDir=targetDir;
        }

        _pdfName=sourceFile;
        _baseName=targetBaseName;

        if (writePict)
        {
            _usePict=true;
            _documentWriter = new BookWriter();
            _imageWriter=new PICTImageWriter();
        }
        else
        {
            _documentWriter = new HTMLWriter();
            _imageWriter=new JPEGImageWriter();

        }

        if (widthParam.isSet())
            _imageWriter.setMaxWidth(widthParam.getValue());
        if (heigthParam.isSet())
            _imageWriter.setMaxHeigth(heigthParam.getValue());
        if (scaleFactorParam.isSet())
            _imageWriter.setScaleFactor(scaleFactorParam.getValue());
        if (qualityParam.isSet())
        {
            if (writePict)
                cl.exitUsageError("'-quality' can only be used when writing JPEG files.");
            _imageWriter.setQuality(qualityParam.getValue());
        }
        
    }

    private void convert()
    {
        //create a directory if it doesn't exist
        File targetPath=new File(_targetDir);
        if (!targetPath.exists())
            targetPath.mkdirs();


        PdfDecoder pdfDecoder=null;

        System.out.println("converting PDF file: "+_pdfName);
        try
        {
            _documentWriter.open(_targetDir+_baseName);
            _documentWriter.writeHeader(_baseName);

            ObjectStore.init("pdfconv");
            pdfDecoder=new PdfDecoder(true);
            pdfDecoder.openPdfFile(_pdfName);
        }
        catch (Exception ex)
        {
            System.err.println("Error while opening PDF file");
            ex.printStackTrace();
            System.exit(1);
        }

        try
        {
            for (int page=1; page<=pdfDecoder.getPageCount(); page++)
            {
                System.out.println("converting page #"+page);
                pdfDecoder.initDisplay(page);
                pdfDecoder.decodePage(true, page);

                String targetName=_baseName+"_page_"+page;
                if (_usePict)
                    targetName+=".pict";
                else
                    targetName += ".jpg";
                _documentWriter.addImage(targetName);
                try
                {
                    BufferedImage targetImage=pdfDecoder.getPageAsImage();
                    if (null==targetImage)
                    {
                        System.err.println("can't render page, skipping");
                    }
                    else
                    {
                        _imageWriter.writeImage(targetImage, _targetDir, targetName);

                    }
                }
                catch (Exception ex)
                {
                    System.err.println("Error while writing image");
                    ex.printStackTrace();
                }
                pdfDecoder.clearPdfImageData();
                System.gc();
            }
            _documentWriter.writeFooter();
        }
        catch (Exception ex)
        {
            pdfDecoder.closePdfFile();
            System.err.println("Error while executing ");
            ex.printStackTrace();
            System.exit(1);
        }
        finally
        {
            _documentWriter.close();
            if (null!=pdfDecoder)
            {
                pdfDecoder.markAllPagesAsUnread();
                pdfDecoder.flushObjectValues(false);
                pdfDecoder.closePdfFile();
            }
            ObjectStore.flush();
        }
    }

}
